function HttpRequest() {
    this.baseUrl = null;
    this.errorMessageSent = false;
    this.sendErrors = true;
    this.handleAs = "text";
    this.userAgent = "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.1 (KHTML, like Gecko) Chrome/21.0.1180.89 Safari/537.1";
    this.parameter = null;
    this.callback = null;
    this.xhr = null;
}

HttpRequest.prototype.setBaseUrl = function(baseUrl) {
    baseUrl && "string" === (typeof baseUrl).toLowerCase() && baseUrl.length > 4 && (this.baseUrl = baseUrl);
    baseUrl = null;
};

HttpRequest.prototype.setParameter = function(parameter) {
    this.parameter = parameter;
    parameter = null;
};

HttpRequest.prototype.setCallback = function(callback) {
    this.callback = callback;
    callback = null;
};

HttpRequest.prototype.handle = function() {
    var parameter = this.parameter;
    parameter || (parameter = {});
    if (!this.baseUrl) {
        this.error({
            error: "Missing base url"
        });
        return;
    }
    if (!Ti.Network || !Ti.Network.online) {
        this.error({
            error: "No connection"
        });
        return;
    }
    var requestUrl = "";
    var encode = function(param) {
        return Ti.Network.encodeURIComponent(param);
    };
    if (parameter) {
        requestUrl += "?";
        for (var paramName in parameter) {
            if ("urls" == paramName) {
                for (var index in parameter.urls) {
                    var url = parameter.urls[index];
                    requestUrl += encode("urls[" + index + "]") + "=";
                    for (var key in url) requestUrl += encode(key) + "%3d" + encode(url[key]) + "%26";
                    requestUrl += "&";
                }
                continue;
            }
            requestUrl += encode(paramName) + "=" + encode(parameter[paramName]) + "&";
        }
    }
    requestUrl = this.baseUrl + requestUrl;
    console.debug("RequestUrl is " + requestUrl, "HttpRequest::handle");
    this.xhr = Ti.Network.createHTTPClient({
        validatesSecureCertificate: false,
        enableKeepAlive: false
    });
    var that = this;
    this.xhr.onload = function() {
        that.load(this);
        that = null;
    };
    this.xhr.onerror = function(e) {
        "undefined" == typeof e && (e = null);
        that.error(e);
        that = null;
    };
    var settings = Alloy.createCollection("AppSettings").settings();
    var timeoutValue = parseInt(settings.httpTimeout(), 10);
    this.xhr.setTimeout(timeoutValue);
    this.xhr.open("GET", requestUrl);
    this.userAgent && this.xhr.setRequestHeader("User-Agent", this.userAgent);
    this.xhr.send({});
    parameter = null;
    settings = null;
};

HttpRequest.prototype.abort = function() {
    if (this.xhr && this.xhr.abort) {
        this.sendErrors = false;
        this.setCallback({}, function() {});
        this.xhr.abort();
        return true;
    }
    return false;
};

HttpRequest.prototype.load = function(xhr) {
    console.debug("Received response " + xhr.responseText, "HttpRequest::load");
    try {
        var response;
        "json" === this.handleAs ? response = JSON.parse(xhr.responseText) : "text" === this.handleAs ? response = xhr.responseText : "xml" === this.handleAs && (response = xhr.responseXML);
    } catch (exception) {
        var tracker = require("Piwik/Tracker");
        tracker.trackException({
            error: exception,
            errorCode: "PiHrLo26"
        });
        this.error({
            error: "Failed to parse response"
        });
        return;
    }
    var isValidResponse = this.isValidResponse(response);
    if (!isValidResponse) {
        response = null;
        xhr = null;
        this.error({
            error: "Invalid response"
        });
        return;
    }
    var callback = this.callback;
    callback || (callback = function() {});
    var parameter = this.parameter;
    try {
        callback.apply(this, [ response, parameter ]);
    } catch (e) {
        console.warn("Failed to call callback method: " + e.message, "HttpRequest::load#callback");
        var tracker = require("Piwik/Tracker");
        tracker.trackException({
            error: e,
            errorCode: "PiHrLo29"
        });
    }
    this.onload && this.onload();
    parameter = null;
    callback = null;
    response = null;
    xhr = null;
    this.cleanup();
};

HttpRequest.prototype.error = function(e) {
    "undefined" == typeof e && (e = null);
    console.warn(e, "HttpRequest::error");
    var L = require("L");
    var message = "";
    var title = L("General_Error");
    var exception = null;
    var errorType = "RequestError";
    var baseUrl = "" + this.baseUrl;
    if (e && e.error || !this.xhr || 200 == this.xhr.status) {
        if (e && e.error) {
            e.error = "" + e.error;
            var originalErrorMessage = e.error;
            (-1 != e.error.indexOf("Host is unresolved") || -1 != e.error.indexOf("Unable to resolve host")) && (e.error = "Host is unresolved");
            -1 != e.error.indexOf("A connection failure occurred") ? e.error = "A connection failure occurred" : -1 != e.error.indexOf("The request timed out") ? e.error = "The request timed out" : -1 != e.error.indexOf("Authentication needed") ? e.error = "Authentication needed" : -1 != e.error.indexOf("Unable to create request") ? e.error = "Unable to create request (bad url?)" : -1 != e.error.indexOf("The request failed because it redirected too many times") ? e.error = "The request failed because it redirected too many times" : -1 != e.error.indexOf("Unable to start HTTP connection") ? e.error = "Unable to start HTTP connection" : -1 != e.error.indexOf("SSL problem") ? e.error = "SSL problem (Possible causes may include a bad/expired/self-signed certificate, clock set to wrong date)" : -1 != e.error.indexOf("Service Temporarily Unavailable") ? e.error = "Service Temporarily Unavailable" : -1 != e.error.indexOf("Internal Server Error") ? e.error = "Internal Server Error" : -1 != e.error.indexOf("Connection reset by peer") ? e.error = "Connection reset by peer" : -1 != e.error.indexOf("Cannot convert host to URI") ? e.error = "Cannot convert host to URI" : -1 != e.error.indexOf("The target server failed to respond") ? e.error = "The target server failed to respond" : -1 != e.error.indexOf("Gateway Time-out") ? e.error = "Gateway Time-out" : -1 != e.error.indexOf("The target server failed to respond") ? e.error = "The target server failed to respond" : -1 != e.error.indexOf("SSL handshake aborted") ? e.error = "SSL handshake aborted: Failure in SSL library, usually a protocol error" : -1 != e.error.indexOf("java.net.SocketTimeoutException") ? e.error = "SocketTimeoutException" : -1 != e.error.indexOf("SSL shutdown failed") ? e.error = "SSL shutdown failed I/O error during system call" : -1 != e.error.indexOf("Connection timed out") ? e.error = "ETIMEDOUT Connection timed out" : -1 != e.error.indexOf("Connect to") && -1 != e.error.indexOf("timed out") ? e.error = "Connect timed out" : -1 != e.error.indexOf("Connect to") && -1 != e.error.indexOf("refused") && (e.error = "Connect refused");
            switch (e.error.toLowerCase()) {
              case "no connection":
                title = L("Mobile_NetworkNotReachable");
                message = L("Mobile_YouAreOffline");
                break;

              case "request aborted":
              case "timeout":
              case "the request timed out":
              case "chunked stream ended unexpectedly":
                message = String.format(L("General_RequestTimedOut"), baseUrl);
                break;

              case "host is unresolved":
              case "not found":
                message = String.format(L("General_NotValid"), baseUrl);
                break;

              case "missing base url":
                break;

              default:
                var statusText = "";
                errorType = e.error;
                exception = originalErrorMessage;
                if (this.xhr && "undefined" != typeof this.xhr.status) {
                    statusText = "" + (this.xhr.statusText ? this.xhr.statusText : this.xhr.status);
                    errorType += "-" + this.xhr.status;
                }
                message = String.format(L("Mobile_NetworkErrorWithStatusCode"), e.error, statusText, baseUrl);
            }
        }
    } else {
        exception = this.xhr.statusText ? this.xhr.statusText : this.xhr.status;
        message = String.format(L("Mobile_NetworkErrorWithStatusCode"), "Unknown", "" + exception, baseUrl);
    }
    if (message && this.displayErrorAllowed()) {
        this.errorMessageSent = true;
        var alertDialog = Ti.UI.createAlertDialog({
            title: title,
            message: message,
            buttonNames: [ L("General_Ok") ]
        });
        alertDialog.show();
        alertDialog = null;
        if (exception) {
            var tracker = require("Piwik/Tracker");
            tracker.trackException({
                error: exception,
                type: errorType,
                file: "Piwik/Network/HttpRequest.js",
                errorCode: "PiHrLe39"
            });
        }
    }
    var callback = this.callback;
    callback || (callback = function() {});
    callback.apply(this, []);
    this.onload && this.onload();
    callback = null;
    e = null;
    this.cleanup();
};

HttpRequest.prototype.displayErrorAllowed = function() {
    if (!this.sendErrors) return false;
    if (this.errorMessageSent) return false;
    if (this.onDisplayErrorAllowed) return this.onDisplayErrorAllowed();
    return true;
};

HttpRequest.prototype.cleanup = function() {
    this.xhr = null;
    this.parameter = null;
    this.callback = null;
    this.onload = null;
};

HttpRequest.prototype.onload = function() {};

HttpRequest.prototype.isValidResponse = function(response) {
    response = null;
    return true;
};

module.exports = HttpRequest;