function getSettings() {
    return Alloy.createCollection("AppSettings").settings();
}

function Tracker() {
    var queue = require("Piwik/Tracker/Queue");
    var trackingConfig = require("alloy").CFG.tracking;
    this.siteId = trackingConfig.siteId;
    this.apiVersion = trackingConfig.apiVersion;
    var documentTitle = "";
    var currentUrl = "";
    var visitCount = 0;
    var uuid = null;
    var baseUrl = trackingConfig.baseUrl;
    var parameter = {};
    this.init = function() {
        visitCount = this._getVisitCount();
        uuid = this._getUniqueId();
        this.prepareVisitCustomVariables();
    };
    this._getUniqueId = function() {
        if (uuid) return uuid;
        var storage = require("Piwik/App/Storage");
        var cachedUUid = storage.get("tracking_visitor_uuid");
        if (cachedUUid && storage.KEY_NOT_FOUND !== cachedUUid) {
            uuid = cachedUUid;
            return uuid;
        }
        return this._generateUniqueId();
    };
    this._generateUniqueId = function() {
        var now = new Date();
        var nowTs = Math.round(now.getTime() / 1e3);
        uuid = "";
        uuid = "mobileweb" + Ti.Platform.id + nowTs + Ti.Platform.model;
        uuid = Ti.Utils.md5HexDigest(uuid).slice(0, 16);
        var storage = require("Piwik/App/Storage");
        storage.set("tracking_visitor_uuid", uuid);
        storage = null;
        return uuid;
    };
    this._getVisitCount = function() {
        if (visitCount) return visitCount;
        var storage = require("Piwik/App/Storage");
        var cachedVisitCount = storage.get("tracking_visit_count");
        if (cachedVisitCount && storage.KEY_NOT_FOUND !== cachedVisitCount) {
            visitCount = cachedVisitCount;
            visitCount++;
        }
        visitCount || (visitCount = 1);
        storage.set("tracking_visit_count", visitCount);
        storage = null;
        return visitCount;
    };
    this.trackWindow = function(title, windowUrl) {
        if (!windowUrl) return;
        var url = "/window/" + windowUrl;
        this.setDocumentTitle(title).setCurrentUrl(url).trackPageView();
    };
    this.trackPageView = function() {
        parameter.action_name = "" + documentTitle;
        parameter.url = currentUrl;
        this._dispatch();
    };
    this.trackEvent = function(event) {
        parameter.action_name = "" + event.title;
        parameter.url = baseUrl + "/event" + event.url;
        event = null;
        this._dispatch();
    };
    this.trackGoal = function(goalId) {
        parameter.idgoal = "" + goalId;
        parameter.url = currentUrl;
        this._dispatch();
    };
    this.trackException = function(exception) {
        if (!exception) return;
        var error = exception.error || null;
        var line = exception.line || "unknown";
        var file = exception.file || "unknown";
        var type = exception.type || "unknown";
        var errorCode = exception.errorCode || "0";
        var message = "" + error;
        var Piwik = require("Piwik");
        if (error && Piwik.isError(error)) {
            error.name && (type = error.name);
            error.sourceURL && (file = error.sourceURL);
            error.line && (line = error.line);
        }
        file = "" + file;
        file && file.length > 60 && (file = file.substr(file.length - 60));
        message && message.length > 200 && (message = message.substr(0, 200));
        var url = "/exception/" + type;
        url += "/" + errorCode;
        url += "/" + file;
        url += "/" + line;
        url += "/" + message;
        var title = "Exception " + type;
        parameter.action_name = "" + title;
        parameter.url = baseUrl + url;
        exception = null;
        this._dispatch();
    };
    this.trackLink = function(sourceUrl, linkType) {
        parameter = {
            url: currentUrl
        };
        parameter[linkType] = sourceUrl;
        this._dispatch();
    };
    this.setDocumentTitle = function(title) {
        documentTitle = "" + title;
        return this;
    };
    this.setCurrentUrl = function(url) {
        currentUrl = baseUrl + url;
        return this;
    };
    this.setCustomVariable = function(index, name, value, scope) {
        var key = "cvar";
        scope && "page" == scope ? key = "cvar" : scope && "visit" == scope && (key = "_cvar");
        parameter[key] || (parameter[key] = {});
        parameter[key]["" + index] = [ "" + name, "" + value ];
    };
    this.prepareVisitCustomVariables = function() {
        var locale = require("Piwik/Locale");
        var Alloy = require("alloy");
        var numAccounts = Alloy.Collections.instance("appAccounts").length;
        this.setCustomVariable(1, "OS", "mobileweb " + Ti.Platform.version, "visit");
        this.setCustomVariable(2, "Piwik Mobile Version", Ti.App.version, "visit");
        this.setCustomVariable(3, "Locale", locale.getPlatformLocale() + "::" + locale.getLocale(), "visit");
        this.setCustomVariable(4, "Num Accounts", numAccounts, "visit");
        locale = null;
    };
    this.isEnabled = function() {
        if (!trackingConfig || !trackingConfig.enabled) return false;
        var settings = getSettings();
        var isEnabled = settings.isTrackingEnabled();
        settings = null;
        return isEnabled;
    };
    this._dispatch = function() {
        if (!this.isEnabled()) {
            queue.clear();
            return;
        }
        this._mixinDefaultParameter();
        queue.offer(parameter);
        parameter = {};
    };
    this._mixinDefaultParameter = function() {
        parameter || (parameter = {});
        var now = new Date();
        parameter.idsite = this.siteId;
        parameter.rand = String(Math.random()).slice(2, 8);
        parameter.h = now.getHours();
        parameter.m = now.getMinutes();
        parameter.s = now.getSeconds();
        parameter.rec = 1;
        parameter.apiv = this.apiVersion;
        parameter.cookie = "";
        parameter.urlref = "http://mobileweb.mobileapp.piwik.org";
        parameter._id = uuid;
        parameter._idvc = visitCount;
        var caps = Ti.Platform.displayCaps;
        parameter.res = caps.platformWidth + "x" + caps.platformHeight;
        parameter._cvar && (parameter._cvar = JSON.stringify(parameter._cvar));
        parameter.cvar && (parameter.cvar = JSON.stringify(parameter.cvar));
    };
    this.askForPermission = function() {
        if (!trackingConfig || !trackingConfig.enabled) return;
        var _ = require("L");
        var alertDialog = Ti.UI.createAlertDialog({
            title: _("Mobile_HelpUsToImprovePiwikMobile"),
            message: _("Mobile_AskForAnonymousTrackingPermission"),
            buttonNames: [ _("General_Yes"), _("General_No") ],
            cancel: 1
        });
        alertDialog.addEventListener("click", function(event) {
            if (!event) return;
            var settings = getSettings();
            switch (event.index) {
              case 0:
                settings.setTrackingEnabled(true);
                Ti.UI.createAlertDialog({
                    message: _("Feedback_ThankYou"),
                    ok: _("General_Ok")
                }).show();
                break;

              default:
                settings.setTrackingEnabled(false);
            }
        });
        alertDialog.show();
    };
    this.init();
}

module.exports = new Tracker();