function Controller() {
    function L(key) {
        return require("L")(key);
    }
    function registerEvents() {
        var session = require("session");
        session.on("websiteChanged", onWebsiteChanged);
    }
    function unregisterEvents() {
        var session = require("session");
        session.off("websiteChanged", onWebsiteChanged);
    }
    function trackWindowRequest() {
        require("Piwik/Tracker").trackWindow("Visitors in Real-time", "visitors-in-real-time");
    }
    function onOpen() {
        trackWindowRequest();
    }
    function onClose() {
        unregisterEvents();
        $.destroy();
        $.off();
    }
    function onWebsiteChanged() {
        require("Piwik/Tracker").trackEvent({
            title: "Website Changed",
            url: "/visitors-in-real-time/change/website"
        });
        doRefresh();
    }
    function openVisitor(event) {
        if (!event || !event.row || !event.row.visitor) return;
        var params = {
            visitor: event.row.visitor
        };
        var visitor = Alloy.createController("visitor", params);
        visitor.open();
    }
    function render(account, counter30Min, counter24Hours, visitorDetails) {
        showReportContent();
        var rows = [];
        counter30Min.title = String.format(L("Live_LastMinutes"), "30");
        var last30minutes = Alloy.createController("live_counter", counter30Min);
        rows.push(last30minutes.getView());
        counter24Hours.title = String.format(L("Live_LastHours"), "24");
        var last24hours = Alloy.createController("live_counter", counter24Hours);
        rows.push(last24hours.getView());
        _.forEach(visitorDetails, function(visitorDetail) {
            var params = {
                account: account,
                visitor: visitorDetail
            };
            var visitorOverview = Alloy.createController("visitor_overview", params);
            var visitorRow = visitorOverview.getView();
            visitorRow.visitor = visitorDetail;
            rows.push(visitorRow);
            visitorRow = null;
        });
        $.liveTable.setData(rows);
        rows = null;
        startRefreshTimer(refreshIntervalInMs);
    }
    function showReportContent() {
        $.loadingindicator.hide();
    }
    function showLoadingMessage() {
        $.loadingindicator.show();
        stopRefreshTimer();
    }
    function onFetchError() {
        console.log("error fetching data");
    }
    function doRefresh() {
        var accountModel = require("session").getAccount();
        var siteModel = require("session").getWebsite();
        if (!accountModel || !siteModel) {
            console.log("account or site not found, cannot refresh live visitors");
            return;
        }
        showLoadingMessage();
        piwikLiveVisitors.fetchVisitors(accountModel, siteModel.id, render, onFetchError);
    }
    function toggleReportConfiguratorVisibility() {
        require("report/configurator").toggleVisibility();
        require("Piwik/Tracker").trackEvent({
            title: "Toggle Report Configurator",
            url: "/visitors-in-real-time/toggle/report-configurator"
        });
    }
    function toggleReportChooserVisibility() {
        require("report/chooser").toggleVisibility();
        require("Piwik/Tracker").trackEvent({
            title: "Toggle Report Chooser",
            url: "/visitors-in-real-time/toggle/report-chooser"
        });
    }
    function close() {
        require("layout").close($.index);
    }
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    arguments[0] ? arguments[0]["__parentSymbol"] : null;
    arguments[0] ? arguments[0]["$model"] : null;
    arguments[0] ? arguments[0]["__itemTemplate"] : null;
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.index = Ti.UI.createWindow({
        barImage: "navbar.png",
        backgroundColor: "#FFFFFF",
        barColor: "#CD1628",
        title: L("Live_VisitorsInRealTime"),
        backButtonTitle: L("Mobile_NavigationBack"),
        id: "index"
    });
    $.__views.index && $.addTopLevelView($.__views.index);
    onClose ? $.__views.index.addEventListener("close", onClose) : __defers["$.__views.index!close!onClose"] = true;
    onOpen ? $.__views.index.addEventListener("open", onOpen) : __defers["$.__views.index!open!onOpen"] = true;
    if (true && !Alloy.isTablet) {
        $.__views.__alloyId85 = Ti.UI.createButton({
            backgroundImage: "ic_action_menu.png",
            width: 32,
            height: 32,
            id: "__alloyId85"
        });
        toggleReportChooserVisibility ? $.__views.__alloyId85.addEventListener("click", toggleReportChooserVisibility) : __defers["$.__views.__alloyId85!click!toggleReportChooserVisibility"] = true;
        $.__views.index.leftNavButton = $.__views.__alloyId85;
    }
    if (true && !Alloy.isTablet) {
        $.__views.__alloyId91 = Ti.UI.createButton({
            backgroundImage: "ic_action_reportmenu.png",
            width: 32,
            height: 32,
            id: "__alloyId91"
        });
        toggleReportConfiguratorVisibility ? $.__views.__alloyId91.addEventListener("click", toggleReportConfiguratorVisibility) : __defers["$.__views.__alloyId91!click!toggleReportConfiguratorVisibility"] = true;
        $.__views.index.rightNavButton = $.__views.__alloyId91;
    }
    $.__views.loadingindicator = Alloy.createController("loading_indicator", {
        rounded: true,
        id: "loadingindicator",
        __parentSymbol: $.__views.index
    });
    $.__views.loadingindicator.setParent($.__views.index);
    $.__views.content = Ti.UI.createView({
        layout: "vertical",
        id: "content"
    });
    $.__views.index.add($.__views.content);
    $.__views.liveTable = Ti.UI.createTableView({
        separatorColor: "#e0e0e0",
        id: "liveTable"
    });
    $.__views.content.add($.__views.liveTable);
    openVisitor ? $.__views.liveTable.addEventListener("click", openVisitor) : __defers["$.__views.liveTable!click!openVisitor"] = true;
    $.__views.countdown = Alloy.createController("countdown_timer", {
        id: "countdown",
        __parentSymbol: $.__views.index
    });
    $.__views.countdown.setParent($.__views.index);
    exports.destroy = function() {};
    _.extend($, $.__views);
    var refreshIntervalInMs = 45e3;
    $.countdown.init(parseInt(refreshIntervalInMs / 1e3, 10));
    var piwikLiveVisitors = Alloy.createCollection("piwikLiveVisitors");
    var refreshTimer = null;
    var stopRefreshTimer = function() {
        refreshTimer && clearTimeout(refreshTimer);
        $.countdown.stop();
    };
    var startRefreshTimer = function(timeoutInMs) {
        stopRefreshTimer();
        $.countdown.init(parseInt(timeoutInMs / 1e3, 10));
        $.countdown.start();
        refreshTimer = setTimeout(function() {
            doRefresh && doRefresh();
        }, timeoutInMs);
    };
    var onResume = function() {
        $.liveTable && $.liveTable.data && $.liveTable.data.length && startRefreshTimer(3e3);
    };
    var onPause = function() {
        stopRefreshTimer && stopRefreshTimer();
    };
    Ti.App.addEventListener("resume", onResume);
    Ti.App.addEventListener("pause", onPause);
    var activity = require("ui/helper").getAndroidActivity($.index);
    if (activity) {
        activity.addEventListener("pause", stopRefreshTimer);
        activity.addEventListener("stop", stopRefreshTimer);
    }
    exports.open = function() {
        registerEvents();
        doRefresh();
        require("layout").open($.index);
    };
    exports.close = close;
    exports.refresh = doRefresh;
    __defers["$.__views.index!close!onClose"] && $.__views.index.addEventListener("close", onClose);
    __defers["$.__views.index!open!onOpen"] && $.__views.index.addEventListener("open", onOpen);
    __defers["$.__views.__alloyId85!click!toggleReportChooserVisibility"] && $.__views.__alloyId85.addEventListener("click", toggleReportChooserVisibility);
    __defers["$.__views.__alloyId87!click!toggleReportChooserVisibility"] && $.__views.__alloyId87.addEventListener("click", toggleReportChooserVisibility);
    __defers["$.__views.__alloyId89!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId89.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.__alloyId91!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId91.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.headerBar!homeIconItemSelected!toggleReportChooserVisibility"] && $.__views.headerBar.on("homeIconItemSelected", toggleReportChooserVisibility);
    __defers["$.__views.headerBar!actionItem1!toggleReportConfiguratorVisibility"] && $.__views.headerBar.on("actionItem1", toggleReportConfiguratorVisibility);
    __defers["$.__views.pullToRefresh!refresh!doRefresh"] && $.__views.pullToRefresh.on("refresh", doRefresh);
    __defers["$.__views.liveTable!click!openVisitor"] && $.__views.liveTable.addEventListener("click", openVisitor);
    _.extend($, exports);
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;