function Controller() {
    function L(key) {
        return require("L")(key);
    }
    function registerEvents() {
        var session = require("session");
        session.on("websiteChanged", onWebsiteChanged);
        session.on("reportDateChanged", onDateChanged);
    }
    function unregisterEvents() {
        var session = require("session");
        session.off("websiteChanged", onWebsiteChanged);
        session.off("reportDateChanged", onDateChanged);
    }
    function trackWindowRequest() {
        var module = reportModel.get("module");
        var action = reportModel.get("action");
        var uniqueId = reportModel.get("uniqueId");
        require("Piwik/Tracker").setCustomVariable(1, "reportModule", module, "page");
        require("Piwik/Tracker").setCustomVariable(2, "reportAction", action, "page");
        require("Piwik/Tracker").setCustomVariable(3, "reportUniqueId", uniqueId, "page");
        require("Piwik/Tracker").trackWindow("Report Without Dimension", "report/without-dimension");
    }
    function onOpen() {
        trackWindowRequest();
    }
    function onClose() {
        unregisterEvents();
        $.destroy();
        $.off();
    }
    function onWebsiteChanged() {
        require("Piwik/Tracker").trackEvent({
            title: "Website Changed",
            url: "/report/without-dimension/change/website"
        });
        doRefresh();
    }
    function onDateChanged(changedReportDate) {
        require("Piwik/Tracker").trackEvent({
            title: "Date Changed",
            url: "/report/without-dimension/change/date"
        });
        reportDate = changedReportDate;
        doRefresh();
    }
    function onReportChosen(chosenReportModel) {
        reportModel = chosenReportModel;
        currentMetric = null;
        doRefresh();
    }
    function showReportContent() {
        $.loadingindicator.hide();
    }
    function showLoadingMessage() {
        $.loadingindicator.show();
    }
    function toggleReportConfiguratorVisibility() {
        require("report/configurator").toggleVisibility();
        require("Piwik/Tracker").trackEvent({
            title: "Toggle Report Configurator",
            url: "/report/without-dimension/toggle/report-configurator"
        });
    }
    function removeAllChildrenFromContent() {
        var children = $.content.children;
        for (var d = children.length - 1; d >= 0; d--) $.content.remove(children[d]);
    }
    function updateWindowTitle(title) {
        $.index.title = title || "";
    }
    function toUnit(size) {
        return size;
    }
    function renderMetricTile(processedReportModel, index) {
        var title = processedReportModel.getTitle();
        var value = processedReportModel.getValue();
        var sortColumnModel = processedReportModel.getSortOrder();
        var sortColumnReport = processedReportCollection.getSortOrder();
        var labelColor = sortColumnReport == sortColumnModel ? "#cb2026" : "black";
        if (0 == index % 2) {
            containerRow = Ti.UI.createView({
                height: Ti.UI.SIZE,
                width: Ti.UI.FILL,
                layout: "horizontal"
            });
            $.content.add(containerRow);
        }
        var outerContainer = Ti.UI.createView({
            height: Ti.UI.SIZE,
            width: "50%"
        });
        var metricContainer = Ti.UI.createView({
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL,
            top: toUnit(22),
            bottom: toUnit(25),
            layout: "vertical"
        });
        var value = Ti.UI.createLabel({
            text: value,
            textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
            top: 0,
            font: {
                fontSize: toUnit(25),
                fontWeight: "bold"
            },
            color: labelColor,
            left: toUnit(10),
            right: toUnit(10),
            height: Ti.UI.SIZE
        });
        var label = Ti.UI.createLabel({
            text: (title + "").toUpperCase(),
            textAlign: Ti.UI.TEXT_ALIGNMENT_CENTER,
            font: {
                fontSize: toUnit(13),
                fontWeight: "bold"
            },
            height: Ti.UI.SIZE,
            top: toUnit(5),
            color: "#7e7e7e",
            left: toUnit(10),
            right: toUnit(10)
        });
        metricContainer.add(value);
        metricContainer.add(label);
        metricContainer.addEventListener("click", function(metric) {
            var changeMetric = function() {
                currentMetric = metric;
                doRefresh();
            };
            return changeMetric;
        }(sortColumnModel));
        outerContainer.add(metricContainer);
        containerRow.add(outerContainer);
        if (1 == index % 2) {
            var horizontalSeparator = Ti.UI.createView({
                height: toUnit(1),
                backgroundColor: "#e6e6e6",
                width: Ti.UI.FILL
            });
            $.content.add(horizontalSeparator);
        }
    }
    function onStatisticsFetched(processedReportCollection) {
        removeAllChildrenFromContent();
        var accountModel = require("session").getAccount();
        updateWindowTitle(processedReportCollection.getReportName());
        showReportContent();
        if (!processedReportCollection) {
            console.error("msising report model");
            return;
        }
        $.reportGraphCtrl.update(processedReportCollection, accountModel);
        processedReportCollection.forEach(renderMetricTile);
    }
    function fixVerticalSeparatorHeight() {
        console.log("attention: this may cause an endless loop because of postlayout event. hopefully you do not see this too often");
        if (!$.content || !$.content.size || !$.content.size.height) return;
        var height = parseInt($.content.size.height, 10);
        height > 5 && setTimeout(function() {
            $.verticalSeparator.setHeight($.content.size.height);
        }, 100);
    }
    function doRefresh() {
        showLoadingMessage();
        var accountModel = require("session").getAccount();
        var siteModel = require("session").getWebsite();
        if (!accountModel || !siteModel) {
            console.log("account or site not found, cannot refresh report without dimension");
            return;
        }
        var module = reportModel.get("module");
        var action = reportModel.get("action");
        var metric = reportModel.getSortOrder(currentMetric);
        processedReportCollection.fetchProcessedReports(metric, {
            account: accountModel,
            params: {
                period: reportDate.getPeriodQueryString(),
                date: reportDate.getDateQueryString(),
                idSite: siteModel.id,
                apiModule: module,
                apiAction: action
            },
            error: function() {
                processedReportCollection.trigger("error", {
                    type: "loadingProcessedReport"
                });
            },
            success: onStatisticsFetched
        });
    }
    function open() {
        $.content.addEventListener("postlayout", fixVerticalSeparatorHeight);
        registerEvents();
        onReportChosen(reportModel);
        require("layout").open($.index);
    }
    function close() {
        require("layout").close($.index);
    }
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    arguments[0] ? arguments[0]["__parentSymbol"] : null;
    arguments[0] ? arguments[0]["$model"] : null;
    arguments[0] ? arguments[0]["__itemTemplate"] : null;
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.index = Ti.UI.createWindow({
        barImage: "navbar.png",
        backgroundColor: "white",
        barColor: "#CD1628",
        zIndex: 3,
        backButtonTitle: L("Mobile_NavigationBack"),
        title: "",
        id: "index"
    });
    $.__views.index && $.addTopLevelView($.__views.index);
    onClose ? $.__views.index.addEventListener("close", onClose) : __defers["$.__views.index!close!onClose"] = true;
    onOpen ? $.__views.index.addEventListener("open", onOpen) : __defers["$.__views.index!open!onOpen"] = true;
    if (true && !Alloy.isTablet) {
        $.__views.__alloyId190 = Ti.UI.createButton({
            backgroundImage: "ic_action_reportmenu.png",
            width: 32,
            height: 32,
            id: "__alloyId190"
        });
        toggleReportConfiguratorVisibility ? $.__views.__alloyId190.addEventListener("click", toggleReportConfiguratorVisibility) : __defers["$.__views.__alloyId190!click!toggleReportConfiguratorVisibility"] = true;
        $.__views.index.rightNavButton = $.__views.__alloyId190;
    }
    $.__views.loadingindicator = Alloy.createController("loading_indicator", {
        rounded: true,
        id: "loadingindicator",
        __parentSymbol: $.__views.index
    });
    $.__views.loadingindicator.setParent($.__views.index);
    $.__views.__alloyId191 = Ti.UI.createScrollView({
        top: 0,
        height: Ti.UI.SIZE,
        contentHeight: "auto",
        id: "__alloyId191"
    });
    $.__views.index.add($.__views.__alloyId191);
    $.__views.__alloyId192 = Ti.UI.createView({
        layout: "vertical",
        height: Ti.UI.SIZE,
        id: "__alloyId192"
    });
    $.__views.__alloyId191.add($.__views.__alloyId192);
    $.__views.reportGraphCtrl = Alloy.createController("graph", {
        id: "reportGraphCtrl",
        __parentSymbol: $.__views.__alloyId192
    });
    $.__views.reportGraphCtrl.setParent($.__views.__alloyId192);
    $.__views.__alloyId193 = Ti.UI.createView({
        backgroundColor: "#dcdcdc",
        height: 1,
        top: 0,
        id: "__alloyId193"
    });
    $.__views.__alloyId192.add($.__views.__alloyId193);
    $.__views.outerContent = Ti.UI.createView({
        height: Ti.UI.SIZE,
        width: Ti.UI.FILL,
        top: 0,
        bottom: 0,
        id: "outerContent"
    });
    $.__views.__alloyId192.add($.__views.outerContent);
    $.__views.content = Ti.UI.createView({
        height: Ti.UI.SIZE,
        layout: "vertical",
        top: 17,
        left: 10,
        right: 10,
        bottom: 10,
        borderWidth: 1,
        borderRadius: 5,
        borderColor: "#e6e6e6",
        id: "content"
    });
    $.__views.outerContent.add($.__views.content);
    $.__views.verticalSeparator = Ti.UI.createView({
        left: "50%",
        zIndex: 2,
        width: 1,
        top: 17,
        backgroundColor: "#e6e6e6",
        height: 0,
        id: "verticalSeparator"
    });
    $.__views.outerContent.add($.__views.verticalSeparator);
    $.__views.__alloyId194 = Ti.UI.createImageView({
        top: 2,
        bottom: 25,
        width: 55,
        height: 19,
        image: "piwik_logo_light_footer.png",
        id: "__alloyId194"
    });
    $.__views.__alloyId192.add($.__views.__alloyId194);
    exports.destroy = function() {};
    _.extend($, $.__views);
    var args = arguments[0] || {};
    var currentMetric = null;
    var reportModel = args.report || false;
    args.reportList || {};
    var reportDate = require("session").getReportDate();
    var processedReportCollection = Alloy.createCollection("piwikProcessedReport");
    var containerRow = null;
    exports.open = open;
    exports.close = close;
    __defers["$.__views.index!close!onClose"] && $.__views.index.addEventListener("close", onClose);
    __defers["$.__views.index!open!onOpen"] && $.__views.index.addEventListener("open", onOpen);
    __defers["$.__views.__alloyId188!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId188.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.__alloyId190!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId190.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.headerBar!back!close"] && $.__views.headerBar.on("back", close);
    __defers["$.__views.headerBar!actionItem1!toggleReportConfiguratorVisibility"] && $.__views.headerBar.on("actionItem1", toggleReportConfiguratorVisibility);
    _.extend($, exports);
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;