function Controller() {
    function L(key) {
        return require("L")(key);
    }
    function open() {
        require("layout").open($.index);
    }
    function close() {
        require("layout").close($.index);
    }
    function onOpen() {
        require("Piwik/Tracker").trackWindow("Visitor Details", "visitor-details");
    }
    function createRow(params) {
        return Alloy.createWidget("org.piwik.tableviewrow", null, params).getRow();
    }
    function createSection(params) {
        return Alloy.createWidget("org.piwik.tableviewsection", null, params).getSection();
    }
    function createOverview(visitor) {
        var visitDateLabel = String.format("%s - %s (%s)", "" + visitor.serverDatePretty, "" + visitor.serverTimePretty, "" + visitor.visitDurationPretty);
        rows.push(createRow({
            title: visitDateLabel,
            className: "visitorTableViewRow"
        }));
        visitor.visitIp && rows.push(createRow({
            title: L("General_VisitorIP"),
            className: "visitorTableViewRow",
            value: visitor.visitIp
        }));
        if (visitor.visitorType) {
            var visitorTypeText = visitor.visitorType;
            switch (visitorTypeText) {
              case "new":
                visitorTypeText = L("General_NewVisitor");
                break;

              case "returning":
                visitorTypeText = "" + visitor.visitorType;
                if (visitor.visitCount) {
                    var visits = "" + parseInt(visitor.visitCount, 10);
                    visits = String.format(L("VisitsSummary_NbVisits"), visits);
                    visitorTypeText += String.format(" (%s)", visits);
                }
            }
            rows.push(createRow({
                title: L("General_VisitType"),
                className: "visitorTableViewRow",
                value: visitorTypeText
            }));
        }
        if (visitor.goalConversions) {
            var goalConversionsText = String.format(L("General_VisitConvertedNGoals"), "" + parseInt(visitor.goalConversions, 10));
            rows.push(createRow({
                title: goalConversionsText,
                className: "visitorTableViewRow"
            }));
        }
        var referrerValue = visitor.referrerName ? visitor.referrerName : visitor.referrerTypeName;
        if (referrerValue) {
            var referrerParams = {
                title: L("General_FromReferrer"),
                className: "visitorTableViewRow"
            };
            if (visitor.referrerUrl) {
                referrerParams.description = visitor.referrerUrl.length > 100 ? visitor.referrerUrl.substr(0, 100) + "..." : visitor.referrerUrl;
                referrerParams.layout = "vertical";
                referrerParams.title += " " + referrerValue;
                referrerParams.focusable = true;
            } else referrerParams.value = referrerValue;
            visitor.referrerKeyword && (referrerParams.title += String.format(": '%s'", "" + visitor.referrerKeyword));
            var referrerRow = createRow(referrerParams);
            referrerRow.addEventListener("click", function() {
                if (visitor.referrerUrl) {
                    require("Piwik/Tracker").trackLink("/visitor/referrer-url", "link");
                    Ti.Platform.openURL(visitor.referrerUrl);
                }
            });
            rows.push(referrerRow);
            referrerRow = null;
        }
        visitor.country && rows.push(createRow({
            title: L("UserCountry_Country"),
            value: visitor.country,
            className: "visitorTableViewRow"
        }));
    }
    function createCustomVariables(visitor) {
        var _ = require("alloy/underscore")._;
        if (_.isEmpty(visitor.customVariables)) return;
        rows.push(createSection({
            title: L("CustomVariables_CustomVariables")
        }));
        for (var customVariableIndex in visitor.customVariables) {
            var customVariable = visitor.customVariables[customVariableIndex];
            var customVariableName = customVariable["customVariableName" + customVariableIndex];
            var customVariableValue = customVariable["customVariableValue" + customVariableIndex];
            rows.push(createRow({
                title: customVariableName,
                className: "visitorTableViewRow",
                value: customVariableValue
            }));
        }
        visitor = null;
    }
    function createSystem(visitor, accessUrl) {
        rows.push(createSection({
            title: L("UserSettings_VisitorSettings")
        }));
        visitor.operatingSystem && rows.push(createRow({
            title: "OS",
            className: "visitorTableViewRow",
            value: visitor.operatingSystem
        }));
        visitor.browserName && rows.push(createRow({
            title: L("UserSettings_ColumnBrowser"),
            className: "visitorTableViewRow",
            value: visitor.browserName
        }));
        var resolution = visitor.resolution;
        resolution && visitor.screenType && "normal" != ("" + visitor.screenType).toLowerCase() && (resolution += String.format(" (%s)", "" + visitor.screenType));
        resolution && rows.push(createRow({
            title: L("UserSettings_ColumnResolution"),
            className: "visitorTableViewRow",
            value: resolution
        }));
        if (visitor.pluginsIcons && visitor.pluginsIcons.length && accessUrl) {
            var row = createRow({
                title: L("UserSettings_Plugins"),
                className: "visitorTableViewRow"
            });
            var right = 10;
            for (var index = 0; visitor.pluginsIcons.length > index; index++) {
                var pluginIcon = visitor.pluginsIcons[index];
                pluginIcon.pluginIcon && row.add(Ti.UI.createImageView({
                    image: accessUrl + pluginIcon.pluginIcon,
                    right: right,
                    width: 14,
                    height: 14,
                    top: 14,
                    className: "pluginIcon"
                }));
                right += 28;
            }
            rows.push(row);
            row = null;
        }
        visitor = null;
    }
    function createActionDetails(visitor, accessUrl) {
        if (!visitor.actionDetails || !visitor.actionDetails.length) {
            visitor = null;
            return;
        }
        var numActions = parseInt(visitor.actions, 10);
        rows.push(createSection({
            title: String.format(L("VisitsSummary_NbActions"), "" + numActions)
        }));
        for (var index = 0; visitor.actionDetails.length > index; index++) {
            var actionDetail = visitor.actionDetails[index];
            if (!actionDetail) continue;
            switch (actionDetail.type) {
              case "action":
                createActionAction(actionDetail, visitor, accessUrl);
                break;

              case "ecommerceOrder":
              case "ecommerceAbandonedCart":
                createEcommerceAction(actionDetail, visitor, accessUrl);
                break;

              default:
                createDefaultAction(actionDetail, visitor, accessUrl);
            }
        }
        visitor = null;
    }
    function createActionAction(actionDetail) {
        var row = Ti.UI.createTableViewRow(helperVisitorActionActionTableViewRow());
        actionDetail.pageTitle && row.add(Ti.UI.createLabel(helperVisitorActionActionPageTitleLabel(actionDetail.pageTitle)));
        actionDetail.url && row.add(Ti.UI.createLabel(helperVisitorActionActionUrlLabel(actionDetail.url)));
        actionDetail.serverTimePretty && row.add(Ti.UI.createLabel(helperVisitorActionActionServerTimeLabel(actionDetail.serverTimePretty)));
        rows.push(row);
        row = null;
        actionDetail = null;
    }
    function createDefaultAction(actionDetail, visitor, accessUrl) {
        var row = Ti.UI.createTableViewRow(helperVisitorActionDefaultTableViewRow());
        var view = Ti.UI.createView(helperVisitorActionDefaultHeadlineView());
        accessUrl && actionDetail.icon && view.add(Ti.UI.createImageView(helperVisitorActionDefaultIconImageView(accessUrl + actionDetail.icon)));
        if (actionDetail.type) {
            var title = "" + actionDetail.type;
            switch (actionDetail.type) {
              case "goal":
                title = L("General_Goal");
                break;

              case "download":
                title = L("General_Download");
                break;

              case "outlink":
                title = L("General_Outlink");
                break;

              case "search":
                title = L("Actions_SubmenuSitesearch");
            }
            view.add(Ti.UI.createLabel(helperVisitorActionDefaultTypeLabel(title)));
        }
        row.add(view);
        view = null;
        actionDetail.url && row.add(Ti.UI.createLabel(helperVisitorActionDefaultUrlLabel(actionDetail.url)));
        rows.push(row);
        row = null;
        actionDetail = null;
    }
    function createEcommerceAction(actionDetail, visitor, accessUrl) {
        var row = Ti.UI.createTableViewRow(helperVisitorActionEcommerceTableViewRow());
        var ecommerceView = Ti.UI.createView(helperVisitorActionEcommerceHeadlineView());
        var ecommerceText = "";
        switch (actionDetail.type) {
          case "ecommerceOrder":
            ecommerceText = L("Goals_EcommerceOrder");
            break;

          case "ecommerceAbandonedCart":
            ecommerceText = L("Goals_AbandonedCart");
            break;

          default:
            ecommerceText = L("Goals_Ecommerce");
        }
        actionDetail.orderId && (ecommerceText = String.format("%s (%s)", "" + ecommerceText, "" + actionDetail.orderId));
        accessUrl && actionDetail.icon && ecommerceView.add(Ti.UI.createImageView(helperVisitorActionEcommerceIconImageView(accessUrl + actionDetail.icon)));
        ecommerceText && ecommerceView.add(Ti.UI.createLabel(helperVisitorActionEcommerceTypeLabel(ecommerceText)));
        var itemDetailsView = Ti.UI.createView(helperVisitorActionEcommerceDetailsView());
        if (actionDetail.itemDetails) for (var index = 0; actionDetail.itemDetails.length > index; index++) {
            var item = actionDetail.itemDetails[index];
            var itemText = "";
            item.itemName && (itemText += item.itemName + " ");
            itemText += String.format("(%s)", "" + item.itemSKU);
            item.itemCategory && (itemText += ", " + item.itemCategory);
            var itemView = Ti.UI.createView(helperVisitorActionEcommerceDetailsItemView());
            itemView.add(Ti.UI.createLabel(helperVisitorActionEcommerceDetailsItemStarLabel(" * ")));
            itemView.add(Ti.UI.createLabel(helperVisitorActionEcommerceDetailsItemNameLabel(itemText)));
            itemDetailsView.add(itemView);
            itemView = null;
            var priceText = "";
            item.price && (priceText += item.price + " " + visitor.siteCurrency);
            item.price && item.quantity && (priceText += " - ");
            item.quantity && (priceText += "Quantity: " + item.quantity);
            itemDetailsView.add(Ti.UI.createLabel(helperVisitorActionEcommerceDetailsPriceLabel(priceText)));
        }
        var revenueText = String.format("%s: %s %s", L("Live_GoalRevenue"), "" + actionDetail.revenue, "" + visitor.siteCurrency);
        actionDetail.revenueSubTotal && (revenueText += String.format(", %s: %s %s", L("General_Subtotal"), "" + actionDetail.revenueSubTotal, "" + visitor.siteCurrency));
        var listOfProductsText = String.format("List of Products (Quantity: %s)", "" + parseInt(actionDetail.items, 10));
        row.add(ecommerceView);
        ecommerceView = null;
        row.add(Ti.UI.createLabel(helperVisitorActionEcommerceRevenueLabel(revenueText)));
        row.add(Ti.UI.createLabel(helperVisitorActionEcommerceDetailsListLabel(listOfProductsText)));
        row.add(itemDetailsView);
        itemDetailsView = null;
        rows.push(row);
        row = null;
        actionDetail = null;
        visitor = null;
    }
    function toUnit(size) {
        return size;
    }
    function helperVisitorActionActionPageTitleLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(10),
            font: {
                fontSize: toUnit(13),
                fontWeight: "bold"
            },
            top: toUnit(5),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionActionUrlLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(10),
            color: "#555555",
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(3),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionActionServerTimeLabel(text) {
        return {
            text: text + "" || "",
            bottom: toUnit(5),
            top: toUnit(3),
            left: toUnit(10),
            font: {
                fontSize: toUnit(13)
            },
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionActionTableViewRow() {
        return {
            layout: "vertical",
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL,
            selectionStyle: 0
        };
    }
    function helperVisitorActionDefaultTableViewRow() {
        return {
            layout: "vertical",
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL,
            selectionStyle: 0
        };
    }
    function helperVisitorActionDefaultHeadlineView() {
        return {
            layout: "horizontal",
            left: toUnit(10),
            top: toUnit(5),
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL
        };
    }
    function helperVisitorActionDefaultIconImageView(image) {
        return {
            image: image,
            left: 0,
            top: toUnit(4),
            width: toUnit(10),
            height: toUnit(9),
            canScale: false,
            enableZoomControls: false
        };
    }
    function helperVisitorActionDefaultTypeLabel(text) {
        return {
            text: text + "" || "",
            top: 0,
            left: toUnit(10),
            font: {
                fontSize: toUnit(13),
                fontWeight: "bold"
            },
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionDefaultUrlLabel(text) {
        return {
            text: text + "" || "",
            color: "#808080",
            bottom: toUnit(5),
            left: toUnit(10),
            top: toUnit(4),
            font: {
                fontSize: toUnit(13)
            },
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceTableViewRow() {
        return {
            layout: "vertical",
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL,
            selectionStyle: 0
        };
    }
    function helperVisitorActionEcommerceHeadlineView() {
        return {
            layout: "horizontal",
            left: toUnit(10),
            top: toUnit(5),
            height: Ti.UI.SIZE,
            width: Ti.UI.FILL
        };
    }
    function helperVisitorActionEcommerceIconImageView(image) {
        return {
            image: image,
            top: toUnit(4),
            left: 0,
            width: toUnit(10),
            height: toUnit(10),
            canScale: false,
            enableZoomControls: false
        };
    }
    function helperVisitorActionEcommerceTypeLabel(text) {
        return {
            text: text + "" || "",
            top: toUnit(1),
            left: toUnit(10),
            font: {
                fontSize: toUnit(13),
                fontWeight: "bold"
            },
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceDetailsView() {
        return {
            layout: "vertical",
            width: Ti.UI.FILL,
            height: Ti.UI.SIZE,
            left: 0,
            bottom: toUnit(5),
            top: toUnit(4)
        };
    }
    function helperVisitorActionEcommerceDetailsItemView() {
        return {
            layout: "horizontal",
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE,
            left: toUnit(10)
        };
    }
    function helperVisitorActionEcommerceDetailsItemStarLabel(text) {
        return {
            text: text + "" || "",
            left: 0,
            color: "#808080",
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(3),
            width: toUnit(15),
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceDetailsItemNameLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(5),
            color: "#808080",
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(3),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceRevenueLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(10),
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(4),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceDetailsListLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(10),
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(4),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    function helperVisitorActionEcommerceDetailsPriceLabel(text) {
        return {
            text: text + "" || "",
            left: toUnit(30),
            color: "#808080",
            font: {
                fontSize: toUnit(13)
            },
            top: toUnit(4),
            width: Ti.UI.SIZE,
            height: Ti.UI.SIZE
        };
    }
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    arguments[0] ? arguments[0]["__parentSymbol"] : null;
    arguments[0] ? arguments[0]["$model"] : null;
    arguments[0] ? arguments[0]["__itemTemplate"] : null;
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.index = Ti.UI.createWindow({
        barImage: "navbar.png",
        backgroundColor: "white",
        barColor: "#CD1628",
        title: L("General_Visitor"),
        id: "index"
    });
    $.__views.index && $.addTopLevelView($.__views.index);
    onOpen ? $.__views.index.addEventListener("open", onOpen) : __defers["$.__views.index!open!onOpen"] = true;
    $.__views.visitorTable = Ti.UI.createTableView({
        id: "visitorTable"
    });
    $.__views.index.add($.__views.visitorTable);
    exports.destroy = function() {};
    _.extend($, $.__views);
    var args = arguments[0] || {};
    var visitor = args.visitor;
    var accountModel = require("session").getAccount();
    var accessUrl = accountModel.getBasePath();
    var rows = [];
    createOverview(visitor, accessUrl);
    createCustomVariables(visitor, accessUrl);
    createSystem(visitor, accessUrl);
    createActionDetails(visitor, accessUrl);
    $.visitorTable.setData(rows);
    rows = null;
    exports.open = open;
    exports.close = close;
    __defers["$.__views.index!open!onOpen"] && $.__views.index.addEventListener("open", onOpen);
    __defers["$.__views.headerBar!back!close"] && $.__views.headerBar.on("back", close);
    _.extend($, exports);
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;