function Controller() {
    function L(key) {
        return require("L")(key);
    }
    function registerEvents() {
        var session = require("session");
        session.on("websiteChanged", onWebsiteChanged);
        session.on("reportDateChanged", onDateChanged);
    }
    function unregisterEvents() {
        var session = require("session");
        session.off("websiteChanged", onWebsiteChanged);
        session.off("reportDateChanged", onDateChanged);
    }
    function openVisitor(event) {
        if (!event || !event.row || !event.row.visitor) return;
        var params = {
            visitor: event.row.visitor
        };
        var visitor = Alloy.createController("visitor", params);
        visitor.open();
    }
    function onWebsiteChanged(website) {
        require("Piwik/Tracker").trackEvent({
            title: "Website Changed",
            url: "/visitor-log/change/website"
        });
        siteModel = website;
        accountModel = require("session").getAccount();
        doRefresh();
    }
    function onDateChanged(date) {
        require("Piwik/Tracker").trackEvent({
            title: "Date Changed",
            url: "/visitor-log/change/date"
        });
        reportDate = date;
        doRefresh();
    }
    function trackWindowRequest() {
        require("Piwik/Tracker").trackWindow("Visitor Log", "visitor-log");
    }
    function onOpen() {
        trackWindowRequest();
    }
    function onClose() {
        unregisterEvents();
        $.destroy();
        $.off();
    }
    function fetchPrevious() {
        showLoadingMessage();
        visitorLog.previous(accountModel, siteModel.id);
        require("Piwik/Tracker").trackEvent({
            title: "Previous Visitors",
            url: "/visitor-log/previous"
        });
    }
    function fetchNext() {
        showLoadingMessage();
        visitorLog.next(accountModel, siteModel.id);
        require("Piwik/Tracker").trackEvent({
            title: "Next Visitors",
            url: "/visitor-log/next"
        });
    }
    function render() {
        showReportContent();
        var rows = [];
        var nextRow = {
            title: L("General_Next"),
            color: "#336699"
        };
        nextRow.left = 10;
        var row = Ti.UI.createTableViewRow(nextRow);
        row.addEventListener("click", fetchNext);
        rows.push(row);
        if (visitorLog.length) visitorLog.forEach(function(visitorDetail) {
            var params = {
                account: accountModel,
                visitor: visitorDetail.attributes
            };
            var visitorOverview = Alloy.createController("visitor_overview", params);
            var visitorRow = visitorOverview.getView();
            visitorRow.visitor = visitorDetail.attributes;
            rows.push(visitorRow);
            visitorRow = null;
        }); else {
            var row = Ti.UI.createTableViewRow({
                title: L("Mobile_NoVisitorFound")
            });
            rows.push(row);
        }
        var prevRow = {
            title: L("General_Previous"),
            color: "#336699"
        };
        prevRow.left = 10;
        var row = Ti.UI.createTableViewRow(prevRow);
        row.addEventListener("click", fetchPrevious);
        rows.push(row);
        $.visitorLogTable.setData(rows);
        rows = null;
    }
    function showReportContent() {
        $.loadingIndicator.hide();
    }
    function showLoadingMessage() {
        $.loadingIndicator.show();
    }
    function doRefresh() {
        showLoadingMessage();
        var period = reportDate.getPeriodQueryString();
        var date = reportDate.getDateQueryString();
        if (!accountModel || !siteModel) {
            console.log("account or site not found, cannot refresh visitor log");
            return;
        }
        visitorLog.initial(accountModel, siteModel.id, period, date);
    }
    function toggleReportConfiguratorVisibility() {
        require("report/configurator").toggleVisibility();
        require("Piwik/Tracker").trackEvent({
            title: "Toggle Report Configurator",
            url: "/visitor-log/toggle/report-configurator"
        });
    }
    function toggleReportChooserVisibility() {
        require("report/chooser").toggleVisibility();
        require("Piwik/Tracker").trackEvent({
            title: "Toggle Report Chooser",
            url: "/visitor-log/toggle/report-chooser"
        });
    }
    function close() {
        require("layout").close($.index);
    }
    require("alloy/controllers/BaseController").apply(this, Array.prototype.slice.call(arguments));
    arguments[0] ? arguments[0]["__parentSymbol"] : null;
    arguments[0] ? arguments[0]["$model"] : null;
    arguments[0] ? arguments[0]["__itemTemplate"] : null;
    var $ = this;
    var exports = {};
    var __defers = {};
    $.__views.index = Ti.UI.createWindow({
        barImage: "navbar.png",
        backgroundColor: "#FFFFFF",
        barColor: "#CD1628",
        title: L("Live_VisitorLog"),
        backButtonTitle: L("Mobile_NavigationBack"),
        id: "index"
    });
    $.__views.index && $.addTopLevelView($.__views.index);
    onClose ? $.__views.index.addEventListener("close", onClose) : __defers["$.__views.index!close!onClose"] = true;
    onOpen ? $.__views.index.addEventListener("open", onOpen) : __defers["$.__views.index!open!onOpen"] = true;
    if (true && !Alloy.isTablet) {
        $.__views.__alloyId224 = Ti.UI.createButton({
            backgroundImage: "ic_action_menu.png",
            width: 32,
            height: 32,
            id: "__alloyId224"
        });
        toggleReportChooserVisibility ? $.__views.__alloyId224.addEventListener("click", toggleReportChooserVisibility) : __defers["$.__views.__alloyId224!click!toggleReportChooserVisibility"] = true;
        $.__views.index.leftNavButton = $.__views.__alloyId224;
    }
    if (true && !Alloy.isTablet) {
        $.__views.__alloyId230 = Ti.UI.createButton({
            backgroundImage: "ic_action_reportmenu.png",
            width: 32,
            height: 32,
            id: "__alloyId230"
        });
        toggleReportConfiguratorVisibility ? $.__views.__alloyId230.addEventListener("click", toggleReportConfiguratorVisibility) : __defers["$.__views.__alloyId230!click!toggleReportConfiguratorVisibility"] = true;
        $.__views.index.rightNavButton = $.__views.__alloyId230;
    }
    $.__views.loadingIndicator = Alloy.createController("loading_indicator", {
        rounded: true,
        id: "loadingIndicator",
        __parentSymbol: $.__views.index
    });
    $.__views.loadingIndicator.setParent($.__views.index);
    $.__views.content = Ti.UI.createView({
        layout: "vertical",
        id: "content"
    });
    $.__views.index.add($.__views.content);
    $.__views.visitorLogTable = Ti.UI.createTableView({
        separatorColor: "#e0e0e0",
        id: "visitorLogTable"
    });
    $.__views.content.add($.__views.visitorLogTable);
    openVisitor ? $.__views.visitorLogTable.addEventListener("click", openVisitor) : __defers["$.__views.visitorLogTable!click!openVisitor"] = true;
    exports.destroy = function() {};
    _.extend($, $.__views);
    var accountModel = require("session").getAccount();
    var siteModel = require("session").getWebsite();
    var reportDate = require("session").getReportDate();
    var visitorLog = Alloy.createCollection("piwikLastVisitDetails");
    visitorLog.on("reset", render);
    exports.open = function() {
        registerEvents();
        doRefresh();
        require("layout").open($.index);
    };
    exports.close = close;
    exports.refresh = doRefresh;
    __defers["$.__views.index!close!onClose"] && $.__views.index.addEventListener("close", onClose);
    __defers["$.__views.index!open!onOpen"] && $.__views.index.addEventListener("open", onOpen);
    __defers["$.__views.__alloyId224!click!toggleReportChooserVisibility"] && $.__views.__alloyId224.addEventListener("click", toggleReportChooserVisibility);
    __defers["$.__views.__alloyId226!click!toggleReportChooserVisibility"] && $.__views.__alloyId226.addEventListener("click", toggleReportChooserVisibility);
    __defers["$.__views.__alloyId228!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId228.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.__alloyId230!click!toggleReportConfiguratorVisibility"] && $.__views.__alloyId230.addEventListener("click", toggleReportConfiguratorVisibility);
    __defers["$.__views.headerBar!homeIconItemSelected!toggleReportChooserVisibility"] && $.__views.headerBar.on("homeIconItemSelected", toggleReportChooserVisibility);
    __defers["$.__views.headerBar!actionItem1!toggleReportConfiguratorVisibility"] && $.__views.headerBar.on("actionItem1", toggleReportConfiguratorVisibility);
    __defers["$.__views.pullToRefresh!refresh!doRefresh"] && $.__views.pullToRefresh.on("refresh", doRefresh);
    __defers["$.__views.visitorLogTable!click!openVisitor"] && $.__views.visitorLogTable.addEventListener("click", openVisitor);
    _.extend($, exports);
}

var Alloy = require("alloy"), Backbone = Alloy.Backbone, _ = Alloy._;

module.exports = Controller;