function appendIndexPhpIfNecessary(accessUrl) {
    accessUrl = appendSlashIfNecessary(accessUrl);
    url.endsWithPhp(accessUrl) || (accessUrl += "index.php");
    return accessUrl;
}

function appendSlashIfNecessary(accessUrl) {
    url.endsWithSlash(accessUrl) || url.endsWithPhp(accessUrl) || (accessUrl += "/");
    return accessUrl;
}

var url = require("url");

var Alloy = require("alloy");

exports.definition = {
    config: {
        columns: {
            accessUrl: "string",
            username: "string",
            tokenAuth: "string",
            name: "string",
            active: "boolean",
            createVersionNumber: "string",
            changeVersionNumber: "string",
            version: "string",
            dateVersionUpdated: "string",
            defaultReport: "string",
            defaultReportDate: "string"
        },
        adapter: {
            type: "properties",
            collection_name: "appaccounts"
        },
        lastUsedAccountPropertyKey: "app_account_id_last_used",
        defaults: {
            active: true,
            createVersionNumber: Ti.App.version,
            changeVersionNumber: Ti.App.version,
            defaultReport: Alloy.CFG.account.defaultReport,
            defaultReportDate: Alloy.CFG.account.defaultReportDate
        }
    },
    extendModel: function(Model) {
        _.extend(Model.prototype, {
            initialize: function() {
                this.on("change:accessUrl", this.completeAccessUrl);
            },
            startWithAllWebsitesDashboard: function() {
                return "MultiSites" === this.get("defaultReport");
            },
            entrySiteId: function() {
                return this.get("defaultReport");
            },
            syncPreferences: function() {},
            getName: function() {
                return this.get("name");
            },
            getDefaultReportDate: function() {
                return this.get("defaultReportDate");
            },
            isSameAccount: function(account) {
                return account && this.get("id") == account.get("id");
            },
            completeAccessUrl: function(accountModel, accessUrl) {
                if (!accessUrl) return;
                accessUrl = appendIndexPhpIfNecessary(accessUrl);
                accountModel.set({
                    accessUrl: accessUrl
                }, {
                    silent: true
                });
            },
            validate: function(attrs) {
                if (!attrs) return "Unknown";
                if (attrs.username && !attrs.password) return "MissingPassword";
                if (!attrs.username && attrs.password) return "MissingUsername";
                var accessUrl = attrs.accessUrl;
                if (!accessUrl || !url.startsWithHttp(accessUrl)) return "InvalidUrl";
            },
            resetPiwikVersion: function() {
                this.set({
                    version: "",
                    dateVersionUpdated: ""
                });
                return this;
            },
            select: function(callback) {
                this.markAccountAsLastUsed();
                this.updatePreferences(callback);
                this.updatePiwikVersion();
            },
            markAccountAsLastUsed: function() {
                var lastUsedAccountKey = this.config.lastUsedAccountPropertyKey;
                var accountId = parseInt(this.id, 10);
                accountId && Ti.App.Properties.setInt(lastUsedAccountKey, accountId);
            },
            updatePreferences: function(callback) {
                if ("anonymous" == this.get("tokenAuth")) {
                    callback(this);
                    callback = null;
                    return;
                }
                var preferences = Alloy.createCollection("piwikAccountPreferences");
                var onSuccess = function(account, defaultReport, defaultReportDate) {
                    account.set("defaultReport", defaultReport);
                    account.set("defaultReportDate", defaultReportDate);
                    callback(account);
                    callback = null;
                };
                var onError = function(account) {
                    callback(account);
                    callback = null;
                };
                preferences.fetchPreferences(this, onSuccess, onError);
            },
            updatePiwikVersion: function() {
                var dateVersionUpdated = this.get("dateVersionUpdated");
                dateVersionUpdated || (dateVersionUpdated = null);
                var dateNow = new Date().toDateString();
                var lastUpdatedDate = new Date(dateVersionUpdated);
                var alreadyUpdatedToday = dateNow == lastUpdatedDate.toDateString();
                if (alreadyUpdatedToday) return;
                var that = this;
                var version = Alloy.createModel("piwikVersion");
                version.fetch({
                    account: this,
                    success: function(model) {
                        that.set({
                            dateVersionUpdated: new Date() + ""
                        });
                        model && model.getVersion() ? that.set({
                            version: "" + model.getVersion()
                        }) : account.version || that.set({
                            version: ""
                        });
                        that.save();
                    },
                    error: function() {}
                });
            },
            getBasePath: function() {
                var accessUrl = this.get("accessUrl");
                if (!accessUrl) return "";
                accessUrl += "";
                if (url.endsWithPhp(accessUrl)) return url.getAbsolutePath(accessUrl);
                accessUrl = appendSlashIfNecessary(accessUrl);
                return accessUrl;
            },
            updateAuthToken: function() {
                var username = this.get("username");
                var password = this.get("password");
                var account = this;
                var onSuccess = function(model, response) {
                    account.set({
                        tokenAuth: response.value
                    });
                };
                var onError = function() {
                    return account.trigger("error", account, "ReceiveAuthTokenError");
                };
                var tokenAuth = Alloy.createModel("piwikTokenAuth");
                tokenAuth.fetchToken(this, username, password, onSuccess, onError);
            }
        });
        return Model;
    },
    extendCollection: function(Collection) {
        _.extend(Collection.prototype, {
            hasAccount: function() {
                return !!this.length;
            },
            lastUsedAccount: function() {
                var lastUsedAccountKey = this.config.lastUsedAccountPropertyKey;
                var hasAccountProperty = Ti.App.Properties.hasProperty(lastUsedAccountKey);
                if (!hasAccountProperty) return this.first();
                var lastUsedAccountId = Ti.App.Properties.getInt(lastUsedAccountKey);
                var lastUsedAccount = this.get(lastUsedAccountId);
                if (!lastUsedAccount) return this.first();
                return lastUsedAccount;
            }
        });
        return Collection;
    }
};

var Alloy = require("alloy"), _ = require("alloy/underscore")._, model, collection;

model = Alloy.M("appAccounts", exports.definition, []);

collection = Alloy.C("appAccounts", exports.definition, model);

exports.Model = model;

exports.Collection = collection;