function plural(word, num) {
    var forms = word.split("_"), minCount = Math.min(pluralRules.length, forms.length), i = -1;
    while (minCount > ++i) if (pluralRules[i](num)) return forms[i];
    return forms[minCount - 1];
}

function relativeTimeWithPlural(number, withoutSuffix, key) {
    var format = {
        mm: "хвилина_хвилини_хвилин_хвилини",
        hh: "година_години_годин_години",
        dd: "день_дня_днів_дня",
        MM: "місяць_місяця_місяців_місяця",
        yy: "рік_року_років_року"
    };
    return "m" === key ? withoutSuffix ? "хвилина" : "хвилину" : number + " " + plural(format[key], +number);
}

function monthsCaseReplace(m, format) {
    var months = {
        nominative: "січень_лютий_березень_квітень_травень_червень_липень_серпень_вересень_жовтень_листопад_грудень".split("_"),
        accusative: "січня_лютого_березня_квітня_травня_червня_липня_серпня_вересня_жовтня_листопада_грудня".split("_")
    }, nounCase = /D[oD]? *MMMM?/.test(format) ? "accusative" : "nominative";
    return months[nounCase][m.month()];
}

function weekdaysCaseReplace(m, format) {
    var weekdays = {
        nominative: "неділя_понеділок_вівторок_середа_четвер_п’ятниця_субота".split("_"),
        accusative: "неділю_понеділок_вівторок_середу_четвер_п’ятницю_суботу".split("_")
    }, nounCase = /\[ ?[Вв] ?(?:попередню|наступну)? ?\] ?dddd/.test(format) ? "accusative" : "nominative";
    return weekdays[nounCase][m.day()];
}

var pluralRules = [ function(n) {
    return 1 === n % 10 && 11 !== n % 100;
}, function(n) {
    return n % 10 >= 2 && 4 >= n % 10 && 0 === n % 10 % 1 && (12 > n % 100 || n % 100 > 14);
}, function(n) {
    return 0 === n % 10 || n % 10 >= 5 && 9 >= n % 10 && 0 === n % 10 % 1 || n % 100 >= 11 && 14 >= n % 100 && 0 === n % 100 % 1;
}, function() {
    return true;
} ];

require("../moment").lang("uk", {
    months: monthsCaseReplace,
    monthsShort: "січ_лют_бер_кві_тра_чер_лип_сер_вер_жов_лис_гру".split("_"),
    weekdays: weekdaysCaseReplace,
    weekdaysShort: "нед_пон_вів_срд_чет_птн_суб".split("_"),
    weekdaysMin: "нд_пн_вт_ср_чт_пт_сб".split("_"),
    longDateFormat: {
        LT: "HH:mm",
        L: "DD.MM.YYYY",
        LL: "D MMMM YYYY г.",
        LLL: "D MMMM YYYY г., LT",
        LLLL: "dddd, D MMMM YYYY г., LT"
    },
    calendar: {
        sameDay: "[Сьогодні в] LT",
        nextDay: "[Завтра в] LT",
        lastDay: "[Вчора в] LT",
        nextWeek: function() {
            return 2 === this.day() ? "[У] dddd [в] LT" : "[В] dddd [в] LT";
        },
        lastWeek: function() {
            switch (this.day()) {
              case 0:
              case 3:
              case 5:
              case 6:
                return "[В минулу] dddd [в] LT";

              case 1:
              case 2:
              case 4:
                return "[В минулий] dddd [в] LT";
            }
        },
        sameElse: "L"
    },
    relativeTime: {
        future: "через %s",
        past: "%s тому",
        s: "декілька секунд",
        m: relativeTimeWithPlural,
        mm: relativeTimeWithPlural,
        h: "годину",
        hh: relativeTimeWithPlural,
        d: "день",
        dd: relativeTimeWithPlural,
        M: "місяць",
        MM: relativeTimeWithPlural,
        y: "рік",
        yy: relativeTimeWithPlural
    },
    ordinal: "%d.",
    week: {
        dow: 1,
        doy: 7
    }
});