function detectPeriodFromReportDate(reportDate) {
    var reportDateToPeriodMapping = {
        week: "week",
        month: "month",
        year: "year"
    };
    if (reportDateToPeriodMapping[reportDate]) return reportDateToPeriodMapping[reportDate];
    return "day";
}

function detectDateFromReportDate(reportDate) {
    var reportDateToDateMapping = {
        week: "today",
        month: "today",
        year: "today"
    };
    if (reportDateToDateMapping[reportDate]) return reportDateToDateMapping[reportDate];
    return reportDate;
}

function reportDate() {
    this.period = null;
    this.date = null;
}

reportDate.prototype.setDate = function(date) {
    this.date = date;
};

reportDate.prototype.setPeriod = function(period) {
    this.period = period;
};

reportDate.prototype.getPeriod = function() {
    return this.period;
};

reportDate.prototype.getPeriodQueryString = function() {
    return this.period;
};

reportDate.prototype.getDateQueryString = function() {
    return this.date;
};

reportDate.prototype.setReportDate = function(reportDate) {
    this.period = detectPeriodFromReportDate(reportDate);
    this.date = detectDateFromReportDate(reportDate);
};

reportDate.prototype.getRangeDate = function() {
    if (!this.date) return [ new Date(), new Date() ];
    var fromDate = null;
    var toDate = null;
    var moment = require("moment/moment");
    if ("last7" == this.date) {
        fromDate = moment().subtract("days", 6).toDate();
        toDate = new Date();
    } else if ("last30" == this.date) {
        fromDate = moment().subtract("days", 29).toDate();
        toDate = new Date();
    } else if ("previous7" == this.date) {
        fromDate = moment().subtract("days", 7).toDate();
        toDate = moment().subtract("days", 1).toDate();
    } else if ("previous30" == this.date) {
        fromDate = moment().subtract("days", 30).toDate();
        toDate = moment().subtract("days", 1).toDate();
    } else if ("week" == this.period) {
        var fromDate = this.toDate(this.date);
        var toDate = this.toDate(this.date);
        if (0 === fromDate.getDay()) {
            fromDate = moment().subtract("days", 7).toDate();
            toDate = moment().subtract("days", 7).toDate();
        }
        fromDate.setDate(fromDate.getDate() - fromDate.getDay() + 1);
        toDate.setDate(toDate.getDate() + (7 - toDate.getDay()));
    } else if (-1 != this.date.indexOf(",")) {
        var fromAndTo = this.date.split(",");
        fromDate = this.toDate(fromAndTo[0]);
        toDate = this.toDate(fromAndTo[1]);
    } else {
        fromDate = this.toDate(this.date);
        toDate = new Date();
    }
    return [ fromDate, toDate ];
};

reportDate.prototype.getAvailableDateRanges = function() {
    var L = require("L");
    var ranges = [ {
        label: L("General_Today"),
        period: "day",
        date: "today"
    }, {
        label: L("General_Yesterday"),
        period: "day",
        date: "yesterday"
    }, {
        label: String.format(L("General_PreviousDaysShort"), "7"),
        period: "range",
        date: "previous7"
    }, {
        label: String.format(L("General_PreviousDaysShort"), "30"),
        period: "range",
        date: "previous30"
    }, {
        label: String.format(L("General_LastDaysShort"), "7"),
        period: "range",
        date: "last7"
    }, {
        label: String.format(L("General_LastDaysShort"), "30"),
        period: "range",
        date: "last30"
    }, {
        label: L("General_CurrentWeek"),
        period: "week",
        date: "today"
    }, {
        label: L("General_CurrentMonth"),
        period: "month",
        date: "today"
    }, {
        label: L("General_CurrentYear"),
        period: "year",
        date: "today"
    } ];
    return ranges;
};

reportDate.prototype.getAvailablePeriods = function() {
    var L = require("L");
    var periods = {
        day: L("CoreHome_PeriodDay"),
        week: L("CoreHome_PeriodWeek"),
        month: L("CoreHome_PeriodMonth"),
        year: L("CoreHome_PeriodYear"),
        range: L("CoreHome_PeriodRange")
    };
    return periods;
};

reportDate.prototype.toDate = function(str) {
    if (!str || "today" == str) return new Date();
    if ("yesterday" == str) {
        var now = new Date();
        now.setDate(now.getDate() - 1);
        return now;
    }
    var changedDate = ("" + str).split("-");
    var formatDate = null;
    changedDate.length >= 3 ? formatDate = new Date(changedDate[0], changedDate[1] - 1, changedDate[2]) : 2 === changedDate.length ? formatDate = new Date(changedDate[0], changedDate[1] - 1, 1) : 1 === changedDate.length && (formatDate = new Date(changedDate[0], 0, 1));
    return formatDate;
};

reportDate.prototype.toPiwikQueryString = function(period, from, to) {
    from || (from = new Date());
    var month = from.getMonth() + 1;
    var year = from.getYear() + 1900;
    var day = from.getDate();
    10 > month && (month = "0" + month);
    var dateString = String(year) + "-" + String(month) + "-" + String(day);
    if ("range" != period) return dateString;
    to || (to = new Date());
    month = to.getMonth() + 1;
    year = to.getYear() + 1900;
    day = to.getDate();
    10 > month && (month = "0" + month);
    dateString = dateString + "," + String(year) + "-" + String(month) + "-" + String(day);
    return dateString;
};

module.exports = reportDate;